package com.mofang.travel.marketing.base.handler;

import com.alibaba.druid.support.json.JSONUtils;
import com.mofang.travel.commons.bean.ResponseResult;
import com.mofang.travel.commons.enums.ResultEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;


@ControllerAdvice
@RestController
public class GlobalExceptionHandler {
    public static final Logger LOGGER = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseResult handleValidException(HttpServletRequest request, Exception e) {
        LOGGER.error("Request Url:{}, message:{}, params:{}", request.getRequestURI(), e.getMessage(), getParams(request));
        return ResponseResult.error(ResultEnum.UNKNOWN_ERROR);
    }

    //    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler(Exception.class)
    public ResponseResult handleBaseException(HttpServletRequest request, Exception e) {
        LOGGER.error("Request Url:{}, params:{}, Exception:{}", request.getRequestURI(), getParams(request), e);
        return ResponseResult.error(ResultEnum.UNKNOWN_ERROR);
    }

    private String getParams(HttpServletRequest request) {
        Map<String, String> map = new HashMap<>();
        Enumeration enu = request.getParameterNames();
        while (enu.hasMoreElements()) {
            String paraName = (String) enu.nextElement();
            map.put(paraName, request.getParameter(paraName));
        }
        return JSONUtils.toJSONString(map);
    }
}