package com.mofang.travel.marketing.base.utils;

import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 处理List工具类
 *
 * @author 刘美丽
 * @date 2022-03-25 10:14
 */
public class ListUtil {
    /**
     * 将传入的list中的指定字段根据 splitChar 分割成列表 (多字段)
     *
     * @param map       key-指定分割后的列表名称 value-字段 lambda写法 Class::getColumn
     * @param list      传入列表
     * @param splitChar 指定分割字符，将根据此字符进行分割
     * @return {@link Map<String,List<String>>} key-列表名称 value-分割后的列表
     * @author 刘美丽
     * @date 2022/03/25 10:27
     */
    public static <T> Map<String, List<String>> splitListByColumn(Map<String, Function<? super T, ? extends String>> map, List<T> list, String splitChar) {
        Map<String, List<String>> result = new HashMap<>();
        map.forEach((k, v) -> {
            result.put(k, list.stream().map(v).flatMap(str -> Arrays.stream(str.split(splitChar))).collect(Collectors.toList()));
        });
        return result;
    }

    /**
     * 将传入的list中的指定字段根据 splitChar 分割成列表 (单字段)
     *
     * @param function  字段
     * @param list      列表
     * @param splitChar 分割字符
     * @return {@link List<String>}
     * @author 刘美丽
     * @date 2022/03/25 10:36
     */
    public static <T> List<String> splitListByColumn(Function<? super T, ? extends String> function, List<T> list, String splitChar) {
        return list.stream()
                .map(function)
                .filter(StringUtils::isNotBlank)
                .flatMap(str -> Arrays.stream(str.split(splitChar)))
                .collect(Collectors.toList());
    }

    /**
     * 统计列表中同一对象出现次数
     *
     * @param list 列表
     * @return {@link Map<T,Integer>} key-对象 value-出现次数
     * @author 刘美丽
     * @date 2022/03/25 10:37
     */
    public static <T> Map<T, Integer> getListCount(List<T> list) {
        Map<T, Integer> result = new HashMap<>();
        for (T t : list) {
            if (!result.containsKey(t)) {
                result.put(t, 0);
            }
            result.put(t, result.get(t) + 1);
        }
        return result;
    }
}
